/*
  LiquidCrystal Library - 1-Key Writer
  
  INSTRUCTIONS
 Printed grid required
   |1|2|3|4|5|6|
  1|_|A|B|C|D|E|
  2|F|G|H|I|J|K|
  3|L|M|N|O|P|Q|
  4|R|S|T|U|V|W|
  5|X|Y|Z|0|1|2|
  6|3|4|5|6|7|8|
  7|9|?|.|,|!|
  
  Hold button to enter ROW - it counts up slowly - lift off to fix value
  Repeat for COLUMN value
  Repeat for ACTION  0 = nothing,  1 = add character,  2 = Clear all,  
                     3 = Dump character  4 = Print to Serial Monitor
  Works quite slowly 
 
 Demonstrates the use a 16x2 LCD display.  The LiquidCrystal
 library works with all LCD displays that are compatible with the 
 Hitachi HD44780 driver. There are many of them out there, and you
 can usually tell them by the 16-pin interface.
 
 This sketch prints a message to the LCD from a single input key
 
  The circuit:
 * LCD RS pin to digital pin 12             
 * LCD Enable pin to digital pin 11  
 * LCD D4 pin to digital pin 5
 * LCD D5 pin to digital pin 4
 * LCD D6 pin to digital pin 3
 * LCD D7 pin to digital pin 2
 * LCD R/W pin to ground
 * 10K resistor:
 * ends to +5V and ground
 * wiper to LCD VO pin (pin 3)
 
 * switch is held HIGH by 10K ohm resistor on pin 8 and
 * goes LOW when pushed
*/
  
// include the library code:
#include <LiquidCrystal.h>

// initialize the library with the numbers of the interface pins
LiquidCrystal lcd(12, 11, 5, 4, 3, 2);

  int switchPin = 8  ;   // Input pin
  int startTime = 0;
  int count =0;        // Row count
  int oldCount = 0;
  int val = 0;
  int gap = 0;
  int oldCol = 0;
  int col =0;          // Column count
  int r = 0;           // Row number  
  int c =0;            // Column number
  int number =0;
  int act = 0;         // Action count
  int oldAct =0;
  int spd = 700;       // Speed adjust - lower = faster
  String slong, x, xx, output;   // Declare strings
  
void setup() {
  // initialize serial communication at 9600 bits per second:
  Serial.begin(9600);
  slong = ("_ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789?.,!");   // Available characters
  x = (""); // Empty string
  output = (""); // Empty string
  // set up the LCD's number of columns and rows: 
  lcd.begin(16, 2);
  // Print a message to the LCD.
  lcd.setCursor(0,0);
  lcd.print("  1-Key Writer");
  delay(1500);
  lcd.setCursor(0,0);
  lcd.print("                ");
  pinMode(switchPin,INPUT);  // set up button/key
  lcd.setCursor(0,0);
  lcd.print("   R:  C:   ");
  }

void loop() { 
  // set the cursor to column 0, line 1
  // (note: line 1 is the second row, since counting begins with 0):
  lcd.setCursor(0, 0);
  while (r == 0 or c == 0) {    // Get Row and Col loop starts here
  
 // Get the ROW number ===========================
  lcd.setCursor(0,0);
  lcd.print("   R:  C:   ");
  val = digitalRead(switchPin);
  while (val == HIGH) {  // Wait until switch closed
    delay(50);           // De-bounce
    val = digitalRead(switchPin);
  }

  startTime = millis();
  delay(50);
  oldCount = 0;
  count = 0;
  lcd.setCursor(5,0);
  lcd.print(col);
  val = digitalRead(switchPin);
  
  while (val == LOW) {   // Wait until switch open
    delay(50);           // De-bounce
    val = digitalRead(switchPin);
    gap = millis() - startTime;
    count = gap/spd;
    if (count != oldCount) {   // Update if row has changed
      lcd.setCursor(0,0);
      lcd.print("   ");
      lcd.setCursor(5,0);
      lcd.print(count); 
      oldCount = count;
      if (count > 7) {startTime = millis();}// Re-start if over range
    }
  }
  
  r = count;
  count = 0;
  oldCount = 0;
  
 // Get the COLUMN number   ================================
  val = digitalRead(switchPin);
  while (val == HIGH) {
    delay(50);
    val = digitalRead(switchPin);
  }

  startTime = millis();
  delay(50);
  oldCol = 0;
  col = 0;
  lcd.setCursor(9,0);
  lcd.print(col);
  val = digitalRead(switchPin);
  
  while (val == LOW) {
    delay(50);
    val = digitalRead(switchPin);
    gap = millis() - startTime;
    col = gap/spd;
    if (col != oldCol) {   // Update if Column has changed
      lcd.setCursor(0,0);
      lcd.print("   ");
      lcd.setCursor(9,0);
      lcd.print(col); 
      oldCol = col;}
      if (col > 6) {startTime = millis();} // Re-set if over range
      if (r == 7 and col > 5 ) {startTime = millis();} // Re-set if over range
  }
  
  lcd.setCursor(0,0);
  c = col;
  col = 0;
  oldCol = 0;
} // ================ End of get Row & Col loop

// Find the character from 'slong' string
  lcd.setCursor(0,0);
  lcd.print("               ");  // Clear panel
  lcd.setCursor(0,0);
  number = (r-1)*6 + c - 1; // Calculate position of character
  if (number >= 42){number = 42;}
  x = x + slong.charAt(number);
  lcd.print(x);  // Display character
//  output = output + x;
  xx = " " + x + "   ";
  delay(700);
  
// Get ACTION   ================================
  lcd.setCursor(0,0);
  lcd.print("   Action" + xx);
  val = digitalRead(switchPin);
  while (val == HIGH) {
    delay(50);   // de-bounce
    val = digitalRead(switchPin);
  }

  startTime = millis();
  delay(50);
  oldAct = 0;
  act = 0;
  lcd.setCursor(0,0);
  lcd.print(act);
  val = digitalRead(switchPin);
  
  while (val == LOW) {
    delay(50);
    val = digitalRead(switchPin);
    gap = millis() - startTime;
    act = gap/950;
    if (act != oldAct) {  //Update if action has changed
      lcd.setCursor(0,0);
      lcd.print("             ");
      lcd.setCursor(0,0);
      lcd.print(act);
      lcd.print(" ");
      if (act == 1) {lcd.print("Add char" + xx);}
      if (act == 2) {lcd.print("Clear all"+ xx);}
      if (act == 3) {lcd.print("Dump char" + xx);}     
      if (act == 4) {lcd.print("Print str      ");}
      if (act > 6) {startTime =millis();} // re-cycle actions
      oldAct = act;}
  }
  if (act == 1) {    // Add new letter ====================
    output = output + x; 
    lcd.setCursor(0,1); 
    lcd.print(output);
    lcd.setCursor(0,0);
    lcd.print("   R:  C:         ");
  }
  if (act == 2) {   // Clear message   =====================
    lcd.setCursor(0,1);
    lcd.print("                ");
    output = "";
    lcd.setCursor(0,0);
    lcd.print("   R:  C:         ");
  }
  if (act == 3) {   // Dump character  =====================
    lcd.setCursor(0,0);
    lcd.print("   R:  C:         ");
  }
  if (act == 4) {   // Print string    =====================
    Serial.println(output); 
    output = ("");
    lcd.setCursor(0,1);
    lcd.print("             ");
    lcd.setCursor(0,0);
    lcd.print("   R:  C:         ");
    }
  lcd.setCursor(0,0);
  c = col;
  col = 0;
  oldCol = 0;
  x = ("");
}
